#!/usr/bin/python
import os
import subprocess
import tempfile
import re
import getpass
import gnupg
import codecs
from lxml import etree
#from jinja2 import Template
import jinja2
from pkg_resources import resource_filename, resource_listdir
from time import time
import treecutter.constants as const
from treecutter.tools import mkdir_p

class Page():
    """Class representing a version of a webpage"""
    def __init__(self,link,page):
        self._link = link
        self._file = page[1]
        self._lang = page[0]
        self._doc = None
        self._resources = []
        self._title = None
        self._menu = None
        self._rendered_article = None

    def language(self):
        return self._lang

    def resources(self):
        return set(self._resources)

    def menu(self):
        return self._menu

    def set_article(self,art):
        self._rendered_article = art

    def prepare(self):
        self._doc = etree.parse(self._file)
        t = self._doc.xpath(u'/db:article/db:info/db:title',namespaces=const.XPATH)
        if t:
            self._title = unicode(t[0].text)
        ta = self._doc.xpath(u'/db:article/db:info/db:titleabbrev',namespaces=const.XPATH)
        if ta:
            self._menu = unicode(ta[0].text)

        dirname = os.path.dirname(self._file)
        cwd = os.getcwd()
        code  = self._doc.xpath(u"//xi:include[@parse='text']",namespaces=const.XPATH)
        if code:
            for c in code:
                href = c.get('href')
                alang = c.get('accept-language')
                xpointer = c.get('xpointer')
                (p, ext) = os.path.splitext(href)
                if ext in const.valid_scripts:
                    exe = []
                    script = os.path.join(os.path.abspath(dirname)+'/'+href)
                    if os.path.isfile(script):
                        exe.append(script)
                    else:
                        if href in resource_listdir('xinclude', ''):
                            script = resource_filename('xinclude', href)
                            exe.append(script)
                        else:
                            print "Script "+href+" in "+self._file+" missing"
                    if alang:
                        exe.append("lang="+alang)
                    if xpointer:
                        exe.append("xptr="+xpointer)
                    print "  executing %15s" % (href),
                    ts = time()
                    os.chdir(dirname)
                    xml = subprocess.Popen(exe,stdout=subprocess.PIPE,
                                           stderr=subprocess.PIPE)
                    (stdout, stderr) = xml.communicate()
                    if stderr:
                        print " ".join(exe)+" ERROR : [ "+stderr+" ]"
                    os.chdir(cwd)
                    te = time()
                    print " [%5.2f s]  (%s)" % (round(te-ts,2),xpointer)
                    xstr = etree.fromstring(stdout)
# inserting the generated code and remove the xinclude reference
                    idp = c.getparent()
                    idp.insert(idp.index(c)+1,xstr)
                    idp.remove(c)

        for r in self._doc.xpath(u"//db:link[@xlink:href]",namespaces=const.XPATH):
            rf = os.path.join(dirname,r.get(const.XLINK+'href'))
            if os.path.isfile(rf):
                if r.get('security')=='encrypt':
                    with open(rf, 'rb') as f:
                        gpg = gnupg.GPG()
                        status = gpg.encrypt_file(
                        f, None, passphrase=getpass.getpass(rf+' password:'), symmetric=True,
                        output=rf+'.gpg')
                    r.set(const.XLINK+'href', r.get(const.XLINK+'href')+'.gpg')
                    rf=rf+'.gpg'
                self._resources.append(rf)
        for i in self._doc.xpath(u"//db:imagedata[@fileref]",namespaces=const.XPATH):
            im = os.path.join(dirname,i.get('fileref'))
            if os.path.isfile(im):
                self._resources.append(im)
            else:
                print "WARNING: File "+im+" is missing!"
        for i in self._doc.xpath(u"//html:form[@action]",namespaces=const.XPATH):
            pyscript = re.split('\.py',i.get('action'),1)[0]+'.py'
            im = os.path.join(dirname,pyscript)
            if os.path.isfile(im):
                self._resources.append(im)

    def render(self, style):

#        xslt_root = etree.XML(open(style+"docbook.xsl", 'r').read())
#        transform = etree.XSLT(xslt_root)
#        result = etree.tostring(transform(xml_root))

        cwd = os.getcwd()
        dirname = os.path.dirname(self._file)
        os.chdir(dirname)
        infile  = os.path.basename(tempfile.mktemp())
        outfile = tempfile.mktemp()
        tfi = open(infile,'w')
        tfi.write(etree.tostring(self._doc,encoding='UTF-8',pretty_print=False))
        tfi.close()
#  cmd = ["saxon-xslt-xinclude","-o",outfile,infile,style_xslt]
        cmd = ["xsltproc","--xinclude","--output",outfile,style+"docbook.xhtml5.xsl",infile]
        retcode = subprocess.call(cmd)
        if retcode:
            print 'Error: '+' '.join(cmd)+' Returncode ['+str(retcode)+']'
        tfo = codecs.open(outfile, "r", "utf-8")
        self._rendered_article = tfo.read()
        tfo.close()
        os.remove(infile)
        os.remove(outfile)
        os.chdir(cwd)

    def template(self,sitemap,style,tdir):
        htmlmenu =  sitemap.gen_menu(self._lang,None,"menu")
        levelmenu = sitemap.gen_menu(self._lang,self,"tree")
        langmenu = sitemap.lang_menu(self._lang,self._link)
        article = self._rendered_article
        templateLoader = jinja2.FileSystemLoader( searchpath="/" )
        templateEnv = jinja2.Environment( loader=templateLoader )

        templateVars = {'title':self._title,
                        'menu':htmlmenu,
                        'article':article,
                        'levelmenu':levelmenu,
                        'langmen':langmenu}

        s = unicode(style, "utf-8")
        t = s+'index.'+self._lang+'.xhtml.tmpl'
        template = templateEnv.get_template( t )
        templateout = template.render( templateVars )

        outfile = tdir+'xhtml'.join(self._file.rsplit('xml',1))
        mkdir_p(os.path.dirname(outfile))
        out = codecs.open(outfile, 'w', 'utf-8')
        out.write(templateout)
        out.close()
