#!/usr/bin/python
# -*- coding: utf-8 -*-

from getpass import getpass
from dateutil.tz import *
from datetime import datetime
from dateutil.parser import parse
import vobject
import httplib
from urlparse import urlparse
from lxml import etree
from lxml.builder import ElementMaker
import sys
import re

from address import Address
from contact import Organization
from treecutter import constants as const


class Events(object):
    def __init__(self, uri):
        self.uri = uri
        self.events = []
        self.data = None
        if uri.scheme == 'file':
            with open(self.uri.path, 'r') as f:
                self.data = f.read()
            f.closed
        if uri.scheme == 'http':
            pw = getpass()
            print "http not yet implemented"
        if uri.scheme == 'https':
            pw = getpass()
            headers = {"User-Agent": "Mozilla/5.0",
                       "Content-Type": "text/xml",
                       "Accept": "text/xml"}

            headers['authorization'] = "Basic %s" % (("%s:%s" % (self.uri.username, pw)).encode('base64')[:-1])
            handle = httplib.HTTPSConnection(self.uri.hostname,self.uri.port)
            res = handle.request('GET', self.uri.path, "", headers)
            r = handle.getresponse()
            if r.status != 200:
                print "Failed to connect! Wrong Password ?"
                sys.exit(5)
            self.data = r.read()
            handle.close()
        headers = ['dtstart','summary','location','description']
        for cal in vobject.readComponents(self.data):
            for ev in cal.vevent_list:
                details = {}
                for k in headers:
                    details[k] = ""
                for p in ev.getChildren():
                    details[p.name.lower()] = p.value
                self.events.append(Event(details))

    def filter(self,query):
        (key, name) = query.split(':')
        fev = []
        if key == 'year':
            for ev in self.events:
                if ev.start.year == int(name):
                    fev.append(ev)
        self.events = fev

    def geocode(self):
        for ev in self.events:
            ev.geocode()

    def sorted(self,order):
        if order == 'start':
            self.events.sort(key=lambda x: x.start, reverse=True)


    def db_xml(self):
        db = ElementMaker(namespace=const.DB_NS, nsmap=const.NSMAP)
        evlist = db.variablelist(db.title(u'Stammtisch träffar'),
                                 role=u'calendar')
        for ev in self.events:
            evlist.append(ev.db_xml())
        return evlist


class Event(object):
    def __init__(self,ev):
        self.start = ev['dtstart']
        self.end = ev['dtend']
        self.summary = ev['summary']
        self.location = ev['location']
        self.org = None
        self.description = ev['description']

    def geocode(self):
        addrlist = self.location.split(',')
        addrfmt = "org,street,postcode,city"
        adict = addrfmt.split(',')
        argdict = dict(zip(adict,addrlist))
        self.org = Organization(argdict['org'])
        del argdict['org']
        addr = Address(**argdict)
        self.org.add_address(addr)
        addr.geocode()


    def db_xml(self):
        db = ElementMaker(namespace=const.DB_NS, nsmap=const.NSMAP)
        # Build paragraphs from the description
        paras = db.listitem(role="description")
        for p in re.split('\n\n',unicode(self.description)):
            paras.append(db.para(p,role="desc"))

        lst = db.varlistentry(
            db.term(db.date(self.start.strftime('%Y %b %d'),role='calendar')),
            db.listitem(db.para(self.summary),db.variablelist(
                    db.varlistentry(
                        db.term("Tid"),
                        db.listitem(db.para(self.start.strftime('%H:%M')))
                        ),
                    db.varlistentry(
                        db.term("Plats"),
                        db.listitem(db.para(self.org.db_xml()))
                        ),
                    db.varlistentry(
                        db.term("Beskrivning"),
                        paras
                )
                    )
                        )
            )
        return lst

# ln = db.link("Text",**{XLINK+"href": "https://"})

if __name__ == "__main__":
    for arg in sys.argv[1:]:
        al = arg.split("=")
        if al[0] == "lang":
            lang = al[1]
        if al[0] == "xptr":
            argument = al[1]

    (uristr,query) = argument.split('|')
    uri = urlparse(uristr)
    events = Events(uri)
    events.filter(query)
    events.geocode()
    events.sorted('start')
    exml = events.db_xml()
    #clean_db(exml)

    #print(etree.tostring(cxml, pretty_print=True))
    #sys.stdout.write(out.encode('utf-8'))
    sys.stdout.write(etree.tostring(exml,encoding='UTF-8',pretty_print=False))
